/*
gpsParser.h 
by TJ Johnson
3/22/09

This diver uses 2 cogs, 1 for FDserial and the other to sort and process the 
gps data.

Call gpsStart(Pin) to initiate the driver.

Then you can call any of the get functions for the specific piece of data 
needed.

Information           Function              Return Data Type
Valid Signal		  getGpsValid()		 	int
Time Stamp      	  getGpsTime()          str pointer
Hours UTC             getGpsTimeHours()     int
Min UTC               getGpsTimeMin()       int
Seconds UTC           getGpsTimeSec()       int
Date Stamp            getGpsDate()          str pointer
Day	 		          getGpsDay()           int
Month			      getGpsMonth()         int
Year                  getGpsYear()          int
Altitude              getGpsAlt()           int
Altitude Tenths       getGpsAltTenths()     int
Altitude              getGpsAltFloat()      float
Heading	 		      getGpsHeading()       int
Heading Tenths        getGpsHeadingTenths() int
Heading			      getGpsHeadingFloat()  float
Speed			      getGpsSpeed()         int
Speed Tenths          getGpsSpeedTenths()   int
Speed                 getGpsSpeedFloat()    float
Latitude		      getGpsLat()		     str pointer
Latitude Deg          getGpsLatDeg()        int +is North - is south
Latitude Min          getGpsLatMin()        int
Latitude Sec          getGpsLatSec()        int
Latitude Decimal Min  getGpsLatDecMin()     float
Longitude             getGpsLong()          str pointer
Longitude Deg         getGpsLongDeg()       int + is east - is west
Longitude Min         getGpsLongMin()       int
Longitude Sec         getGpsLongSec()       int
Longitude Decimal Min getGpsLongDecMin()    float
        

The standard int functions for Altitude, Heading, Speed return a truncated
result. The tenths function have been provided to get the remainder. 
Example Heading is 163.4
       getHeading() returns 163
	   getHeadngTenths() returns 4
	    

For use with the Parallax GPS unit /raw pin should be connected to ground.		
WARNING : A 1k resistor must be placed in line with the SIO signal, 
 		  when connecting the Parallax GPS unit.
		
WARNING : If the GPS Signal is invalid, there is a high chance that the altitude 
result will return trash, please protect your program accordingly.
	
	
See end of file for MIT License.		
*/
#ifndef __GPSPARSER_H_
#define __GPSPARSER_H_




// function prototypes

   // Name: gpsStart
   // Input: input pin
   // Output: cogId the parser is running in
   // Dependices: FdSerial.c
   // Process: Start a string parsing system that updates based on incomming 
   	  // gps data, use get****** to get a sepecific value.
   int gpsStart(const int Pin);
  
   // Name: gpsStop
   // Dependicies: none
   // Process: Stops the driver frees a cog.
   void gpsStop(void);

   // Name: getData
   // Dependicies: decodeGPRMC, decodeGPGGA, FdSerial_rx
   // Process: Starts a loop looking for the string ID and calls appropiate 
   	  //decode funciton  
   static void getData(void);

   // Name: decodeGPRMC
   // Dependicies: FdSerial_rx()
   // Process: parses throught the incomming serial data sorting to the correct 
   	  // data field  
   static void decodeGPRMC(void);

   // Name: decodeGPGGA
   // Dependicies: FdSerial_rx
   // Process: parses throught the incomming serial data sorting to the correct 
   	  // data field   
   static void decodeGPGGA(void);
   
   // Name: getGpsValid
   // Input: none
   // Output: 1 for a valid signal, 0 if not
   // Dependicies: none
   // Process: Returns 1 if the data is valid otherwise 0
   int getGpsValid(void);
   
   // Name: getGpsTime
   // Input: none
   // Output: pointer to UTC time string
   // Dependicies: none
   // Process: returns the pointer to the current UTC time String 
   char* getGpsTime(void);
  
   // Name: getGpsTimeHours
   // Input: none
   // Output: int current time hours
   // Dependicies: none
   // Process: returns the hours of the current time 
   int getGpsTimeHours(void);
   
   
     // Name: getGpsTimeMin
   // Input: none
   // Output: int current time minutes
   // Dependicies: none
   // Process: returns the minutes of the current time 
   int getGpsTimeMin(void);
   
     // Name: getGpsTimeSec
   // Input: none
   // Output: int current time in seconds
   // Dependicies: none
   // Process: returns the seconds of the current time 
   int getGpsTimeSec(void);
   
   // Name:getGpsData
   // Input:none
   // Output:The pointer the string containg the date stamp
   // Dependicies: none
   // Process: Returns the pointer to the date stamp string. 
   char* getGpsDate(void);
  
   // Name: getGpsDay
   // Input: none
   // Output: day of the timeStamp as an int.
   // Dependicies: none
   // Process: Returns interger value of the day from the dateStamp. 
   int getGpsDay(void);
 
   // Name: getGpsMonth
   // Input: none
   // Output: month of timestamp as an int.
   // Dependicies: none
   // Process: Returns interger value of the month from the dateStamp. 
   int getGpsMonth(void); 
   
   // Name: getGpsYear
   // Input: none
   // Output: year of timeStamp as an int.
   // Dependicies: none
   // Process: Returns interger value of the year from the dateStamp. 
   int getGpsYear(void);

   // Name: getGpsLat
   // Input: none
   // Output: pointer to the string conting the lat
   // Dependicies: none
   // Process: returns a pointer to the string parsed from the GPS. 
   char* getGpsLat(void);
      
   // Name: getGpsLatDeg
   // Input: none
   // Output: Degress the current latitude
   // Dependicies: getGpsDeg
   // Process: calls getGpsDeg with the string from lat to return the current 
   	  //deg value. 
   int getGpsLatDeg(void);

   // Name: getGpsLatMin
   // Input: none
   // Output: Current Minutes of Latitude
   // Dependicies:getGpsMin
   // Process: calls getGpsMin with the lat string to return the current min value  
   int getGpsLatMin(void);

   // Name: getGpsLatSec
   // Input: none
   // Output: Current Seconds of Latitude
   // Dependicies: getGpsSec
   // Process: Calls getGpsSec with the lat string to return the current
   	  // second value 
   int getGpsLatSec(void);
    
   // Name:getGpsLatDecMin
   // Input: none
   // Output: floating point value of latitude in decimal minutes
   // Dependicies: getGpsDecMin
   // Process: calls getGpsDecMin to getGps the decimal minutes of the latitude string 
   float getGpsLatDecMin(void);
  
   // Name: getGpsLong
   // Input: none
   // Output: pointer to the string conting the long
   // Dependicies: none
   // Process: returns a pointer to the string parsed from the GPS.
  char* getGpsLong(void);
  
   // Name: getGpsLongDeg
   // Input: none
   // Output: Degress the current longitude
   // Dependicies: getGpsDeg
   // Process: calls getGpsDeg with the string from long to return the current 
   	  //deg value.  
   int getGpsLongDeg(void);
  
   // Name: getGpsLongMin
   // Input: none
   // Output: Current Minutes of Longitude
   // Dependicies:getGpsMin
   // Process: calls getGpsMin with the long string to return the current min value 
   int getGpsLongMin(void);
  

   // Name: getGpsLongSec
   // Input: none
   // Output: Current Seconds of Longitude
   // Dependicies: getGpsSec
   // Process: Calls getGpsSec with the long string to return the current
   	  // second value 
   int getGpsLongSec(void); 
  
   // Name:getGpsLongDecMin
   // Input: none
   // Output: floating point value of longitude in decimal minutes
   // Dependicies: getGpsDecMin
   // Process: calls getGpsDecMin to getGps the decimal minutes of the longitude string  
   float getGpsLongDecMin(void); 
  
   // Name: getGpsAlt
   // Input: none
   // Output: The current altitude in meters
   // Dependicies: atoi
   // Process: converts the altitude string to a interger number 
   int getGpsAlt(void);

   // Name: getGpsAltTenths
   // Input: none
   // Output: The tents of a meter of the current altitude
   // Dependicies: none
   // Process: getGpss the last digit of the altitude and returns it as an int. 
   int getGpsAltTenths(void);
   
   // Name: getGpsAltFloat
   // Input: none
   // Output: Current altitude as an float
   // Dependicies: atoi
   // Process: convert the altitude to an float value 
   float getGpsAltFloat(void);
   
   // Name: getGpsHeadingDeg
   // Input: none
   // Output: Current heading as an int
   // Dependicies: atoi
   // Process: convert the heading to an int value 
   int getGpsHeadingDeg(void);

   // Name: getGpsHeadingTenths
   // Input: none
   // Output: the tenths of the current heading as a int
   // Dependicies: none
   // Process: Returns the tenths of the current heading. 
   int getGpsHeadingTenths(void);

   // Name: getGpsHeadingFloat
   // Input: none
   // Output: the current heading as a floating point num.
   // Dependicies: atof
   // Process: convert the heading string to a float 
   float getGpsHeadingFloat(void);

   // Name: getGpsSats
   // Input: none
   // Output: Number of Satalites in view
   // Dependicies: none
   // Process: Returns the current number of statlites in view. 
   int getGpsSats(void);
    
   // Name: getGpsSpeed
   // Input: none
   // Output: the speed in knots
   // Dependicies: atoi
   // Process: Return the current speed as an int. 
   int getGpsSpeed(void);
  
   // Name: getGpsSpeedTenths
   // Input: none
   // Output: the tents of the current speed as an int
   // Dependicies: none
   // Process: returns the tents value of the current speed. 
   int getGpsSpeedTenths(void);

   // Name: getGpsSpeedFloat
   // Input: none
   // Output: the current speed as a floating point number
   // Dependicies: atof
   // Process: converts the current speed string to a floating point value  
   float getGpsSpeedFloat(void);

   // Name: getGpsDeg
   // Input: char array 
   // Output: the current Deg as an int
   // Dependicies: atoi, charloc
   // Process: finds the decimal place in the string and works backwards to 
   	  //find the degrees
   static int getDeg(char *array);

   // Name: getGpsMin
   // Input: char array
   // Output: the current minutes as an int
   // Dependicies: atoi,charLoc
   // Process: finds the minutes in the string working backwards from the
   	  // decimal place
   static int getMin(char *array);

   // Name: getGpsDecMin
   // Input: char array 
   // Output: the floating point value of the decimal minutes
   // Dependicies: charLoc, atof
   // Process: removes the deg from the string and converts the remainder
   	  // to a float 
   static float getDecMin(char *array);

   // Name: getGpsDecSec
   // Input: char array
   // Output: The interger of the seconds  
   // Dependicies: charLoc, atoi
   // Process: converts the remainder after the decimal to an interger 
   	  //and devides by 60 to convert to seconds 
   static int getDecSec(char *array);

   // Name: charLoc
   // Input: char array, char to find
   // Output: the index of the char to find
   // Dependicies: none
   // Process: looks through the array for the seach char returning the index. 
   static int charLoc(char *array, char searchChar);
#endif
/*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/